import java.io.*;

import SecureBlackbox.Base.SBCustomFSAdapter;
import SecureBlackbox.Base.TElVFSOutputFileHandle;
import SecureBlackbox.Base.TSBFileShareMode;

public class FTPSFileStream extends TElVFSOutputFileHandle {
	String m_path;
	RandomAccessFile m_stream;
	int m_errCode;
	
	public FTPSFileStream(String path, int openMode, TSBFileShareMode shareMode) {
		try {
			m_path = path;
			m_stream = new RandomAccessFile(path, getStringMode(openMode, shareMode));
		} catch (Exception e) {
			e.printStackTrace();
			Main.Log(e.getMessage(), true);
		}
	}
	
	private String getStringMode(int OpenMode, TSBFileShareMode ShareMode) throws Exception {
		String result = "";
        if ((OpenMode & SBCustomFSAdapter.fileOpenTruncate) != 0)
        {
			result = "rw";
        }
        else if ((OpenMode & SBCustomFSAdapter.fileOpenAppend) != 0)
        {
			result = "rw";
        }
        else if ((OpenMode & SBCustomFSAdapter.fileOpenCreate) != 0)
        {
			result = "rw";
        }
        else if (((OpenMode & SBCustomFSAdapter.fileOpenRead) != 0) &&
            ((OpenMode & SBCustomFSAdapter.fileOpenWrite) == 0))
        {
			result = "r";
        }
        else if (((OpenMode & SBCustomFSAdapter.fileOpenRead) == 0) &&
            ((OpenMode & SBCustomFSAdapter.fileOpenWrite) != 0))
        {
			result = "rw";
		}
        else if (((OpenMode & SBCustomFSAdapter.fileOpenRead) != 0) &&
            ((OpenMode & SBCustomFSAdapter.fileOpenWrite) != 0))
        {
			result = "rw";
		}

		
		switch (checkAccessMode(m_path, ShareMode)){
			case 1:
				if (result.compareTo("r") == 0)
					result = "rw";
		}

		return result;
	}

	public String getFileName() {
		return new File(m_path).getName();
	}

	public String getPath() {
		return m_path;
	}

	public long getLength() throws IOException {
		return m_stream.length();
	}

	public void setPosition(long pos) throws IOException {
		m_stream.seek(pos);
	}

	public int getErrorCode() {
		return m_errCode;
	}

	protected int checkAccessMode(String realpath, TSBFileShareMode ShareMode) throws Exception {
		int result = -1;
        
		if (TSBFileShareMode.ssmExclusive == ShareMode) 
            result = 0;
        if (TSBFileShareMode.ssmDenyRead == ShareMode)
        	result = 1;

        if (TSBFileShareMode.ssmDenyWrite == ShareMode)
        	result = 0;

        if (TSBFileShareMode.ssmNone == ShareMode)
            result = 1;
        
		return result;
	}

	public void write(byte[] buffer, int offset, int count) {
		try {
			m_stream.write(buffer, offset, count);
		} catch (IOException e) {
			e.printStackTrace();
			Main.Log(e.getMessage(), true);
		}
	}

	public void close() {
		try {
			m_stream.close();
		} catch (IOException e) {
			e.printStackTrace();
			Main.Log(e.getMessage(), true);
		}
	}

	public int read(byte[] buffer, int bufferOffset, int count) {
		try {
			return m_stream.read(buffer, bufferOffset, count);
		} catch (IOException e) {
			e.printStackTrace();
			Main.Log(e.getMessage(), true);
		}
		return 0;
	}

	public long GetPosition() {
		try {
			return m_stream.getFilePointer();
		} catch (IOException e) {
			e.printStackTrace();
			Main.Log(e.getMessage(), true);
		}
		return 0;
	}

	public long GetSize() {
		try {
			return m_stream.length();
		} catch (IOException e) {
			e.printStackTrace();
			Main.Log(e.getMessage(), true);
		}
		return 0;
	}

	public void SetLength(long newSize) {
		try {
			m_stream.setLength(newSize);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

}
