

import java.awt.BorderLayout;
import java.awt.FlowLayout;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.border.EmptyBorder;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;
import javax.swing.filechooser.FileNameExtensionFilter;
import javax.swing.SwingConstants;

import SecureBlackbox.Base.JNI;
import SecureBlackbox.Base.SBConstants;
import SecureBlackbox.Base.SBUtils;
import SecureBlackbox.Base.TElFileStream;
import SecureBlackbox.Office.SBOffice;
import SecureBlackbox.Office.SBOfficeSecurity;
import SecureBlackbox.Office.TElOfficeBinaryRC4CryptoAPIEncryptionHandler;
import SecureBlackbox.Office.TElOfficeBinaryRC4EncryptionHandler;
import SecureBlackbox.Office.TElOfficeDocument;
import SecureBlackbox.Office.TElOfficeOpenXMLAgileEncryptionHandler;
import SecureBlackbox.Office.TElOfficeOpenXMLPasswordKeyEncryptor;
import SecureBlackbox.Office.TElOfficeOpenXMLStandardEncryptionHandler;
import SecureBlackbox.Office.TSBOfficeDocumentFormat;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;


public class Main extends JDialog {

	private static final long serialVersionUID = 1L;
	private final JPanel contentPanel = new JPanel();
	private JTextField tbSource;
	private JLabel lbDetails;
	private JButton btnEncryptTo;
	private JButton btnDecryptTo;
	private JButton btnSign;
	private JButton btnVerify;

	private BinEncOptionsForm frmBinEncOptions = null;
	private OpenXMLEncOptionsForm frmOpenXMLEncOptions = null;
	private TElOfficeDocument _OfficeDocument = null;
	private JPanel gbInfo;

	/**
	 * Launch the application.
	 */
	public static void main(String[] args) {
		try {
			Main dialog = new Main();
			dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
			dialog.setVisible(true);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Create the dialog.
	 */
	public Main() {
		addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent e) {
				if (_OfficeDocument != null)
					_OfficeDocument.close();
			}
		});
		
		setTitle("Secure office");
		setBounds(100, 100, 521, 346);
		getContentPane().setLayout(new BorderLayout());
		contentPanel.setBorder(new EmptyBorder(5, 5, 5, 5));
		getContentPane().add(contentPanel, BorderLayout.CENTER);
		contentPanel.setLayout(null);
		
		JLabel lblSourceFile = new JLabel("Source file:");
		lblSourceFile.setBounds(10, 11, 172, 14);
		contentPanel.add(lblSourceFile);
		
		tbSource = new JTextField();
		tbSource.setBounds(10, 31, 237, 20);
		contentPanel.add(tbSource);
		tbSource.setColumns(10);
		
		JButton btnBrowse = new JButton("Browse");
		btnBrowse.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				browseSourceClick();
			}
		});
		btnBrowse.setBounds(267, 30, 95, 23);
		contentPanel.add(btnBrowse);
		
		JButton btnLoadDocument = new JButton("Load document...");
		btnLoadDocument.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				loadDocumentClick();
			}
		});
		btnLoadDocument.setBounds(372, 30, 131, 23);
		contentPanel.add(btnLoadDocument);
		
		gbInfo = new JPanel();
		gbInfo.setBorder(new TitledBorder(null, "Info", TitledBorder.LEADING, TitledBorder.TOP, null, null));
		gbInfo.setBounds(10, 62, 493, 213);
		contentPanel.add(gbInfo);
		gbInfo.setLayout(null);
		
		lbDetails = new JLabel("<html><body>Document is not loaded</body></html>");
		lbDetails.setVerticalAlignment(SwingConstants.TOP);
		lbDetails.setBounds(55, 27, 428, 175);
		gbInfo.add(lbDetails);
		{
			JPanel buttonPane = new JPanel();
			buttonPane.setLayout(new FlowLayout(FlowLayout.RIGHT));
			getContentPane().add(buttonPane, BorderLayout.SOUTH);
			
			btnEncryptTo = new JButton("Encrypt To...");
			btnEncryptTo.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					encryptToClick();
				}
			});
			buttonPane.add(btnEncryptTo);
			
			btnDecryptTo = new JButton("Decrypt To...");
			btnDecryptTo.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					decryptToClick();
				}
			});
			buttonPane.add(btnDecryptTo);
			{
				btnSign = new JButton("Sign");
				btnSign.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						signClick();
					}
				});
				btnSign.setActionCommand("OK");
				buttonPane.add(btnSign);
				getRootPane().setDefaultButton(btnSign);
			}
			{
				btnVerify = new JButton("Verify");
				btnVerify.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						verifyClick();
					}
				});
				btnVerify.setActionCommand("Cancel");
				buttonPane.add(btnVerify);
			}
		}
		
		init();
	}

	protected void verifyClick() {
        if (!_OfficeDocument.getIsSigned())
        {
            showErrorMessage("The document is not signed");
            return;
        }

        SignaturesInfoForm frmSignaturesInfo = new SignaturesInfoForm();
        frmSignaturesInfo.setDocument(_OfficeDocument);
        frmSignaturesInfo.setVisible(true);
	}

	protected void signClick() {
        if (!_OfficeDocument.getSignable())
        {
            showErrorMessage("Cannot sign document");
            return;
        }

        SignatureOptionsForm frmSignatureOptions = new SignatureOptionsForm();
        frmSignatureOptions.setDocument(_OfficeDocument);
        frmSignatureOptions.setVisible(true);

        try
        {
            _OfficeDocument.flush();
        }
        catch (Exception ex)
        {
            showErrorMessage("Failed to flush document: " + ex.getMessage());
            ex.printStackTrace();
        }

        updateDocumentInfo();
	}

	protected void encryptToClick() {
        String DestFilename = "";
        String Ext = extension(tbSource.getText());

        if (_OfficeDocument.getDocumentFormat() == TSBOfficeDocumentFormat.dfBinary)
        {
            frmBinEncOptions.setSaveDefaultExt(Ext);
            frmBinEncOptions.setModal(true);
            frmBinEncOptions.setVisible(true);
            if (!frmBinEncOptions.isOK())
                return;

            _OfficeDocument.setEncryptionHandler(frmBinEncOptions.getEncryptionHandler());
            DestFilename = frmBinEncOptions.getDestText();
        }
        else if (_OfficeDocument.getDocumentFormat() == TSBOfficeDocumentFormat.dfOpenXML)
        {
            frmOpenXMLEncOptions.setSaveDefaultExt(Ext);
            frmOpenXMLEncOptions.setModal(true);
            frmOpenXMLEncOptions.setVisible(true);
            if (!frmOpenXMLEncOptions.isOK())
                return;

            _OfficeDocument.setEncryptionHandler(frmOpenXMLEncOptions.getEncryptionHandler());
            DestFilename = frmOpenXMLEncOptions.getDestText();
        }
        else
        {
            showErrorMessage("Document cannot be encrypted");
            return;
        }

        TElFileStream F = new TElFileStream(DestFilename, "rw", true);
        try
        {
            _OfficeDocument.encryptTo(F);
        }
        catch (Exception ex)
        {
            showErrorMessage("Failed to encrypt document: " + ex.getMessage());
            ex.printStackTrace();
            return;
        }
        finally
        {
            F.Free();
        }

        showMessage("The document encrypted OK");
	}

	protected void decryptToClick() {
		if (!_OfficeDocument.getIsEncrypted())
		{
			showMessage("The document is not encrypted");
			return;
		}

		if (_OfficeDocument.getEncryptionHandler() instanceof TElOfficeOpenXMLStandardEncryptionHandler)
		{
			String Password = requestPassword("Please enter a document open password:");
			if (Password.length() == 0)
				return;

			((TElOfficeOpenXMLStandardEncryptionHandler)_OfficeDocument.getEncryptionHandler()).setPassword(Password);
			if (!((TElOfficeOpenXMLStandardEncryptionHandler)_OfficeDocument.getEncryptionHandler()).isPasswordValid())
			{
				showErrorMessage("Invalid password");
				return;
			}
		}
		else if (_OfficeDocument.getEncryptionHandler() instanceof TElOfficeOpenXMLAgileEncryptionHandler)
		{
			String Password = requestPassword("Please enter a document open password:");
			if (Password.length() == 0)
				return;

			boolean Found = false;
			TElOfficeOpenXMLAgileEncryptionHandler AgileHandler = (TElOfficeOpenXMLAgileEncryptionHandler)_OfficeDocument.getEncryptionHandler();
			for (int i = 0; i < AgileHandler.getKeyEncryptorCount(); i++)
			{
				if (AgileHandler.getKeyEncryptor(i).getClass().getName().indexOf("TElOfficeOpenXMLPasswordKeyEncryptor") >= 0)
				{
					((TElOfficeOpenXMLPasswordKeyEncryptor)AgileHandler.getKeyEncryptor(i)).setPassword(Password);
					if (((TElOfficeOpenXMLPasswordKeyEncryptor)AgileHandler.getKeyEncryptor(i)).isPasswordValid())
					{
						Found = true;
						break;
					}
				}
			}

			if (!Found)
			{
				showErrorMessage("Invalid password");
				return;
			}
		}
		else if (_OfficeDocument.getEncryptionHandler() instanceof TElOfficeBinaryRC4EncryptionHandler)
		{
			String Password = requestPassword("Please enter a document open password:");
			if (Password.length() == 0)
				return;

			((TElOfficeBinaryRC4EncryptionHandler)_OfficeDocument.getEncryptionHandler()).setPassword(Password);
			if (!((TElOfficeBinaryRC4EncryptionHandler)_OfficeDocument.getEncryptionHandler()).isPasswordValid())
			{
				showErrorMessage("Invalid password");
				return;
			}
		}
		else if (_OfficeDocument.getEncryptionHandler() instanceof TElOfficeBinaryRC4CryptoAPIEncryptionHandler)
		{
			String Password = requestPassword("Please enter a document open password:");
			if (Password.length() > 0)
				return;

			((TElOfficeBinaryRC4CryptoAPIEncryptionHandler)_OfficeDocument.getEncryptionHandler()).setPassword(Password);
			if (!((TElOfficeBinaryRC4CryptoAPIEncryptionHandler)_OfficeDocument.getEncryptionHandler()).isPasswordValid())
			{
				showErrorMessage("Invalid password");
				return;
			}
		}
		else
		{
			showErrorMessage("Unknown encryption handler");
			return;
		}

		String Ext = extension(tbSource.getText());
		if (Ext.length() > 0)
			Ext = Ext.substring(1);

		String saveFileName = getSaveFileName(Ext);
		if (saveFileName.length() == 0)
			return;

		TElFileStream F = new TElFileStream(saveFileName, "rw", true);
		try
		{
			_OfficeDocument.decryptTo(F);
		}
        catch (Exception ex)
        {
            showErrorMessage("Failed to decrypt document: " + ex.getMessage());
            ex.printStackTrace();
            return;
        }
        finally
		{
			F.Free();
		}

		showMessage("The document decrypted OK");
	}

	String getSaveFileName(String ext){
		JFileChooser fc = new JFileChooser();

		FileNameExtensionFilter filter = new FileNameExtensionFilter(String.format("Current document type *.%s | All files (*.*)|*.*", ext), ext, "*");
	    fc.setFileFilter(filter);		 

	    int returnVal = fc.showSaveDialog(this);
		if (returnVal == JFileChooser.APPROVE_OPTION)
	        return fc.getSelectedFile().getPath();

	    return "";
	}
	
	public String extension(String fullPath) {
	    int dot = fullPath.lastIndexOf('.');
	    return fullPath.substring(dot + 1);
	}


	void showMessage(String msg) {
		JOptionPane.showMessageDialog(null, msg, getTitle(), JOptionPane.INFORMATION_MESSAGE);
	}	

    private String requestPassword(String cap) {
		JPasswordField jpf = new JPasswordField();
		int result = JOptionPane.showConfirmDialog(null, jpf, cap, JOptionPane.OK_CANCEL_OPTION);
		if (result == 0)
			return new String(jpf.getPassword());
		return "";
	}

	protected void loadDocumentClick() {
		try
		{
			_OfficeDocument.open(tbSource.getText(), false);
		}
        catch (Exception ex)
        {
            showErrorMessage("Failed to load document: " + ex.getMessage());
            ex.printStackTrace();
            return;
        }
        finally
		{
			updateDocumentInfo();
		}		
	}

	private void updateDocumentInfo()
	{
		String s = "<html><body>Document format: " + SBOffice.documentFormatToString(_OfficeDocument.getDocumentFormat()) + "<br>";
		if (_OfficeDocument.getBinaryDocument() != null)
			s = s + "Document type: " + SBOffice.binaryDocumentTypeToString(_OfficeDocument.getBinaryDocument().getDocumentType()) + "<br>";
		else if (_OfficeDocument.getOpenXMLDocument() != null)
			s = s + "Document type: " + SBOffice.openXMLDocumentTypeToString(_OfficeDocument.getOpenXMLDocument().getDocumentType()) + "<br>";

		gbInfo.setEnabled((_OfficeDocument.getDocumentFormat() != TSBOfficeDocumentFormat.dfUnknown));

		s = s + "\r\n";
		if (_OfficeDocument.getIsEncrypted())
		{
			if (_OfficeDocument.getEncryptionHandler() != null)
				s = s + "The document is ENCRYPTED using '" + _OfficeDocument.getEncryptionHandler().getClass().getName() + "' handler<br>";
			else
				s = s + "The document is ENCRYPTED using UNKNOWN handler<br>";

			if (_OfficeDocument.getEncryptionHandler() instanceof TElOfficeOpenXMLStandardEncryptionHandler)
			{
				s = s + " - Encryption algorithm: " +
					SBConstants.getAlgorithmNameByAlgorithm(((TElOfficeOpenXMLStandardEncryptionHandler)_OfficeDocument.getEncryptionHandler()).getEncryptionAlgorithm()) + "<br>";
			}
			else if (_OfficeDocument.getEncryptionHandler() instanceof TElOfficeOpenXMLAgileEncryptionHandler)
			{
				s = s + " - Key Encryptors: " +
					((TElOfficeOpenXMLAgileEncryptionHandler)_OfficeDocument.getEncryptionHandler()).getKeyEncryptorCount() + "<br>";
				s = s + " - Cipher algorithm: " +
					SBConstants.getAlgorithmNameByAlgorithm(((TElOfficeOpenXMLAgileEncryptionHandler)_OfficeDocument.getEncryptionHandler()).getCipherAlgorithm()) + "<br>";
				s = s + " - Hash algorithm: " +
					SBConstants.getAlgorithmNameByAlgorithm(((TElOfficeOpenXMLAgileEncryptionHandler)_OfficeDocument.getEncryptionHandler()).getHashAlgorithm()) + "<br>";
			}
			else if (_OfficeDocument.getEncryptionHandler() instanceof TElOfficeBinaryRC4CryptoAPIEncryptionHandler)
			{
				s = s + " - Key length: " +
					((TElOfficeBinaryRC4CryptoAPIEncryptionHandler)_OfficeDocument.getEncryptionHandler()).getKeyLength() + "<br>";
				if (((TElOfficeBinaryRC4CryptoAPIEncryptionHandler)_OfficeDocument.getEncryptionHandler()).getEncryptDocumentProperties())
					s = s + " - Document properties are encrypted<br>";
				else
					s = s + " - Document properties are NOT encrypted<br>";
			}
		}

		if (_OfficeDocument.getIsSigned())
		{
			s = s + "Document is SIGNED<br>";
		}

		if (!_OfficeDocument.getIsEncrypted() && !_OfficeDocument.getIsSigned())
			s = s + "Document is neither encrypted nor signed<br>";

		if (_OfficeDocument.getOpenXMLDocument() != null)
		{
			if (_OfficeDocument.getOpenXMLDocument().getSignatureLineCount() > 0)
				s = s + String.format("Document contains %s signature line(s)<br>", _OfficeDocument.getOpenXMLDocument().getSignatureLineCount());
		}
		else if (_OfficeDocument.getOpenXPSDocument() != null)
		{
			if (_OfficeDocument.getOpenXPSDocument().getSignatureDefinitionCount() > 0)
				s = s + String.format("Document contains %s signature definiotion(s)<br>", _OfficeDocument.getOpenXPSDocument().getSignatureDefinitionCount());
		}

		s += "</body></html>";
		lbDetails.setText(s);
		btnDecryptTo.setEnabled(_OfficeDocument.getIsEncrypted());
		btnEncryptTo.setEnabled(_OfficeDocument.getEncryptable());
		btnSign.setEnabled(_OfficeDocument.getSignable());
		btnVerify.setEnabled(_OfficeDocument.getIsSigned());
	}
	
	static void showErrorMessage(String msg){
		JOptionPane.showMessageDialog(null, msg, "Error", JOptionPane.ERROR_MESSAGE);
	}

	protected void browseSourceClick() {
		String fileName = getOpenFileName();
		if (fileName.length() > 0)
			tbSource.setText(fileName);	}

	String getOpenFileName(){
		JFileChooser fc = new JFileChooser();
	    int returnVal = fc.showOpenDialog(this);
		if (returnVal == JFileChooser.APPROVE_OPTION)
	        return fc.getSelectedFile().getPath();

	    return "";
	} 

	private void init() {
		try
		{
			#error Please pick the evaluation license key from <SecureBlackbox>\LicenseKey.txt file and place it here. If the evaluation key expires, you can request an extension using the form on https://www.eldos.com/sbb/keyreq/
			SBUtils.setLicenseKey("<put the key here>");
	
			SBOfficeSecurity.initialize();

			// Initialize SecureBlackbox JNI library. sbbjni32.dll and sbbjni64.dll must be located in java.library.path
			// You can also call JNI.initialize("full_path_to_dll") method.
			JNI.initialize();
			
			frmBinEncOptions = new BinEncOptionsForm();
			frmOpenXMLEncOptions = new OpenXMLEncOptionsForm();
	
			_OfficeDocument = new TElOfficeDocument();
		} catch(Exception ex) {
			ex.printStackTrace();
		}
	}
}
