import java.util.Vector;

import javax.swing.table.AbstractTableModel;

import SecureBlackbox.Base.SBConstants;
import SecureBlackbox.Base.TElX509Certificate;


public class CertTableModel extends AbstractTableModel {
	private static final long serialVersionUID = 2771284071470299948L;
	public static final int S_INDEX = 0;
	public static final int I_INDEX = 1;
	public static final int VF_INDEX = 2;
	public static final int VT_INDEX = 3;
	public static final int A_INDEX = 4;
	
	protected Vector<CertDataRow> dataVector = new Vector<CertDataRow>();
	
    protected String[] columnNames = {
    		"Subject",
    		"Issuer", 
    		"Valid From",
    		"Valid To",
    		"Algorithm", 
    };
    
	public String getColumnName(int column) {
        return columnNames[column];
    }
    
	public int getColumnCount() {
		return columnNames.length;
	}

	public int getRowCount() {
		return dataVector.size();
	}

    public Object getValueAt(int row, int col) {
        CertDataRow record = (CertDataRow)dataVector.get(row);
        switch (col) {
        	case S_INDEX:
        		return record.Subject;
        	case I_INDEX:
	        	 return record.Issuer;
            case VF_INDEX:
                return record.VFrom;
            case VT_INDEX:
                return record.VTo;                
            case A_INDEX:
               return record.Alg;
            default:
               return new Object();
        }
    }

	public void clear() {
		dataVector.removeAllElements();
	}

	public void addRow(TElX509Certificate Cert) {
		// Subject
		String s = Cert.getSubjectName().CommonName;
		if (s == "")
			s = Cert.getSubjectName().Organization;

		
		// Issuer
		String i = Cert.getIssuerName().CommonName;
		if (i == "")
			i = Cert.getIssuerName().Organization;
		
		// Validity period
		String vf = Cert.getValidFrom().toString();
		String vt = Cert.getValidTo().toString();
		
		// Algorithm
		String a = GetAlgStr(Cert.getPublicKeyAlgorithm());
		a = a + " (" + Cert.getPublicKeySize() + " bits)";
		
    	dataVector.addElement(new CertDataRow(s, i, vf, vt, a));
    	fireTableDataChanged();
	}
	
	private String GetAlgStr(int alg)
	{
		switch (alg)
		{
			case SBConstants.SB_CERT_ALGORITHM_ID_RSA_ENCRYPTION: return "RSA";
			case SBConstants.SB_CERT_ALGORITHM_ID_DSA: return "DSA";
			case SBConstants.SB_CERT_ALGORITHM_DH_PUBLIC: return "DH";
			default: return "Unknown";				
		}
	}
}