

import java.awt.EventQueue;

import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JMenuBar;
import javax.swing.JMenu;
import javax.swing.JOptionPane;
import javax.swing.JPasswordField;
import javax.swing.JSeparator;
import javax.swing.JPanel;
import javax.swing.JTextField;

import java.awt.BorderLayout;
import java.awt.Dimension;
import javax.swing.JLabel;
import javax.swing.JComboBox;
import javax.swing.JButton;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;

import SecureBlackbox.Base.*;
import SecureBlackbox.PKI.*;

import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;


public class Main extends JDialog {
	private static final long serialVersionUID = 1L;
	private JTable lvCerts;
	private CertTableModel model;

	TElPKCS11CertStorage Storage;
	TElPKCS11SessionInfo Session;
	private JComboBox<String> cbSlots;
	private JButton btnAddCert;
	private JButton btnRemoveCert;
	private JButton btnOpen;
	private JMenu mnuCloseStorage;
	private JButton btnProcessFiles;
	ProcessorForm frmProcessor;
	/**
	 * Launch the application.
	 */
	public static void main(String[] args) {
		EventQueue.invokeLater(new Runnable() {
			public void run() {
				try {
					Main dialog = new Main();
					dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
					dialog.setVisible(true);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}

	/**
	 * Create the dialog.
	 */
	public Main() {
		#error Please pick the evaluation license key from <SecureBlackbox>\LicenseKey.txt file and place it here. If the evaluation key expires, you can request an extension using the form on https://www.eldos.com/sbb/keyreq/
		SBUtils.setLicenseKey("<put the key here>");
		
    JNI.initialize();
  
		setTitle("CryptoToken (PKCS#11) Demo Application");
		setBounds(100, 100, 655, 438);
		
		JMenuBar menuBar = new JMenuBar();
		setJMenuBar(menuBar);
		
		JMenu mnFile = new JMenu("File");
		menuBar.add(mnFile);
		
		JMenu mnOpenStorage = new JMenu("Open Storage");
		mnOpenStorage.addMouseListener(new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent e) {
				openStorageClick();
			}
		});
		mnFile.add(mnOpenStorage);
		
		JMenu mnOpenNssStorage = new JMenu("Open NSS Storage");
		mnOpenNssStorage.addMouseListener(new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent e) {
				openNSSStorageClick();
			}
		});
		mnFile.add(mnOpenNssStorage);
		
		mnuCloseStorage = new JMenu("Close Storage");
		mnuCloseStorage.addMouseListener(new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent e) {
				if (Storage != null)
					CloseStorage();				
			}
		});
		mnFile.add(mnuCloseStorage);
		
		JSeparator separator = new JSeparator();
		mnFile.add(separator);
		
		JMenu mnExit = new JMenu("Exit");
		mnExit.addMouseListener(new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent e) {
				exit();
			}
		});
		mnFile.add(mnExit);
		
		JMenu mnHelp = new JMenu("Help");
		menuBar.add(mnHelp);
		
		JMenu mnAbout = new JMenu("About...");
		mnAbout.addMouseListener(new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent e) {
				About form = new About();
				form.setModal(true);
				form.setVisible(true);
			}
		});
		mnHelp.add(mnAbout);
		
		JPanel panel = new JPanel();
		panel.setPreferredSize(new Dimension(10, 60));
		getContentPane().add(panel, BorderLayout.NORTH);
		panel.setLayout(null);
		
		JLabel lblAvailableSlots = new JLabel("Available slots:");
		lblAvailableSlots.setBounds(10, 11, 150, 14);
		panel.add(lblAvailableSlots);
		
		cbSlots = new JComboBox<String>();
		cbSlots.setBounds(10, 26, 150, 22);
		panel.add(cbSlots);
		
		btnOpen = new JButton("Open");
		btnOpen.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				try {
					OpenSession();
				} catch (Exception e1) {
					e1.printStackTrace();
				}
			}
		});
		btnOpen.setBounds(170, 26, 80, 23);
		panel.add(btnOpen);
		
		btnAddCert = new JButton("Add Certificate");
		btnAddCert.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				try {
					AddCertificate();
				} catch (Exception e1) {
					e1.printStackTrace();
				}
			}
		});
		btnAddCert.setBounds(260, 26, 117, 23);
		panel.add(btnAddCert);
		
		btnRemoveCert = new JButton("Remove Certificate");
		btnRemoveCert.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (lvCerts.getSelectedRowCount() > 0)
					RemoveCertificate();				
			}
		});
		btnRemoveCert.setBounds(387, 26, 135, 23);
		panel.add(btnRemoveCert);
		
		btnProcessFiles = new JButton("Process Files");
		btnProcessFiles.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (lvCerts.getSelectedRowCount() > 0)
					ProcessFiles();				
			}
		});
		btnProcessFiles.setBounds(532, 26, 105, 23);
		panel.add(btnProcessFiles);
		
		JScrollPane scrollPane = new JScrollPane();
		getContentPane().add(scrollPane, BorderLayout.CENTER);
		
		model = new CertTableModel();
		lvCerts = new JTable(model);
		lvCerts.setFillsViewportHeight(true);
		scrollPane.setViewportView(lvCerts);

		model.addTableModelListener(new TableModelListener() {
			public void tableChanged(TableModelEvent e) {
				lvCerts_SelectedIndexChanged();
			}
		});
		
	}

	protected void exit() {
		if (Storage != null)
			CloseStorage();
		
		dispose();
	}

	protected void openNSSStorageClick() {
        if (Storage != null)
            CloseStorage();
        OpenNSSStorage();
	}

	protected void openStorageClick() {
		if (Storage != null)
			CloseStorage();
		OpenStorage();
	}
	
	String getOpenFileName(){
		JFileChooser fc = new JFileChooser();

	    int returnVal = fc.showOpenDialog(this);
		if (returnVal == JFileChooser.APPROVE_OPTION)
	        return fc.getSelectedFile().getPath();

	    return "";
	}

	protected String getOpenDir() {
		JFileChooser fc = new JFileChooser();
		fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
	    int returnVal = fc.showOpenDialog(this);
		if (returnVal == JFileChooser.APPROVE_OPTION)
	        return fc.getSelectedFile().getPath();

	    return "";
	}
	
	private void OpenStorage()
	{
		TElPKCS11SlotInfo SlotInfo;
		String fileName = getOpenFileName();
		if (fileName.length() > 0)
		{
			Storage = new TElPKCS11CertStorage();
			Storage.setDLLName(fileName);

			model.clear();
			cbSlots.removeAllItems();
			cbSlots.addItem("<Please select a slot>");

			try
			{
				Storage.open();

				for (int i = 0; i < Storage.getModule().getSlotCount(); i++)
				{
					SlotInfo = Storage.getModule().getSlot(i);
					cbSlots.addItem(SlotInfo.getSlotDescription());
				}

				cbSlots.setSelectedIndex(0);
			}
			catch (Exception E)
			{
				Storage.Free();
				showErrorMessage("Error opening storage: " + fileName + "\r\n" + E.getMessage());
				return;
			}

			SetupButtons();
		}
	}

    private void showErrorMessage(String msg) {
    	JOptionPane.showMessageDialog(null, msg, getTitle(), JOptionPane.ERROR_MESSAGE);
	}
	
    private void OpenNSSStorage()
    {
        TElPKCS11SlotInfo SlotInfo;
		String fileName = getOpenFileName();
		if (fileName.length() > 0)
        {
			String folderName = getOpenDir();
			if (folderName.length() > 0)
	        {			
                Storage = new TElPKCS11CertStorage();
                Storage.setDLLName(fileName);

                Storage.getNSSParams().setUseNSSMode(true);
                Storage.getNSSParams().setConfigDir(folderName.replace("\\", "/"));

                model.clear();
                cbSlots.removeAllItems();
                cbSlots.addItem("<Please select a slot>");

                try
                {
                    Storage.open();

                    for (int i = 0; i < Storage.getModule().getSlotCount(); i++)
                    {
                        SlotInfo = Storage.getModule().getSlot(i);
                        cbSlots.addItem(SlotInfo.getSlotDescription());
                    }

                    cbSlots.setSelectedIndex(0);
                }
                catch (Exception E)
                {
                    Storage.Free();
                    if (E.getMessage().indexOf("SecureBlackbox_PKCS11Proxy") > 0)
                    {
                        showErrorMessage(E.getMessage() + "\r\n\r\n\r\n" + "SecureBlackbox_PKCS11Proxy.dll is an unmanaged proxy DLL. It can be found in the folder, to which you installed SecureBlackbox. Copy it to the folder in which your executable file is located.");
                    }
                    else
                        showErrorMessage("Error opening storage: " + fileName + "\r\n" + E.getMessage());
                    return;
                }

                SetupButtons();
            }
        }
    }
    
	private void CloseStorage()
	{
		Storage.close();
		cbSlots.removeAllItems();
		model.clear();
		Storage.Free();
		if (Session != null) 
			Session = null;

		SetupButtons();
	}	
	
	private void SetupButtons()
	{
		btnAddCert.setEnabled(Session != null);
		btnRemoveCert.setEnabled(Session != null);
		mnuCloseStorage.setEnabled(Storage != null);
		btnOpen.setEnabled(Storage != null);
	}

 	private String requestPassword() {
		JPasswordField jpf = new JPasswordField();
		int result = JOptionPane.showConfirmDialog(null, jpf, "Please enter password for certificate", JOptionPane.OK_CANCEL_OPTION);
		if (result == 0)
			return new String(jpf.getPassword());
		return "";
	}	

	private void AddCertificate() throws Exception
	{
		TElX509Certificate Cert;
		TElFileStream F;
		int R;
		String Pass;
		String fileName = getOpenFileName();
		if (fileName.length() > 0)
		{
			F = new TElFileStream(fileName, "rw", true);
			try
			{
				Pass = requestPassword();
				if (Pass.length() > 0)
				{
					Cert = new TElX509Certificate();
					try
					{
						R = Cert.loadFromStreamPFX(F, Pass, 0);
					}
					catch(Exception ex)
					{
						Cert.Free();
						throw ex;
					}
					if (R == 0)
					{
						try
						{
							Storage.add(Cert, true);
						}
						finally
						{
							Cert.Free();
						}
						RefreshCertificates();
					}
					else
						showErrorMessage("Failed to load certificate, error " + R);
				}
			}
			finally
			{
				F.Free();
			}
		}
	}

	private void RemoveCertificate()
	{
		String S;
		TElX509Certificate Cert;
		if ((lvCerts.getSelectedRowCount() > 0) && (lvCerts.getSelectedRow() < Storage.getCount()))
		{
			Cert = Storage.getCertificate(lvCerts.getSelectedRow());
			S = "Subject: " + Cert.getSubjectName().CommonName + "\r\nIssuer: " + Cert.getIssuerName().CommonName;
			if (chooseMessage("The following certificate will be deleted:\r\n" + S))
			{
				Storage.remove(lvCerts.getSelectedRow());
				RefreshCertificates();
			}
		}
	}
	
	private boolean chooseMessage(String msg) {
		return (JOptionPane.showConfirmDialog(this, msg) == JOptionPane.YES_OPTION);
	}

	private void OpenSession() throws Exception
	{
		String Pin;
		Boolean RO;
		// Checking whether we can establish a new session 
		if (cbSlots.getSelectedIndex() <= 0)
			return;

		if (!Storage.getModule().getSlot(cbSlots.getSelectedIndex() - 1).getTokenPresent())
		{
			showErrorMessage("Token not found in specified slot");
			return;
		}

		// Closing current session
		if (Session != null)
		{
			model.clear();
			Session.logout();
			Storage.closeSession(0);
			Session = null;
		}

		// Opening new session
		RO = Storage.getModule().getSlot(cbSlots.getSelectedIndex() - 1).getReadOnly();
		try
		{
			Session = Storage.openSession(cbSlots.getSelectedIndex() - 1, RO);
		}
		catch (Exception ex)
		{
			if (!RO)
				Session = Storage.openSession(cbSlots.getSelectedIndex() - 1, true);
			else
				throw ex;
		}

		Pin = requestPIN();
		if ((Session != null) && (Pin.length() > 0))
		{
			try
			{
				Session.login((int) SBPKCS11Base.utUser, Pin);
			}
			catch(Exception ex)
			{
				Storage.closeSession(0);
				Session = null;
			}
		}
		RefreshCertificates();
		SetupButtons();
	}

	private void RefreshCertificates()
	{
		TElX509Certificate Cert;
		model.clear();
		for (int i = 0; i < Storage.getCount(); i++)
		{
			Cert = Storage.getCertificate(i);
			model.addRow(Cert);
		}
	}

	private void ProcessFiles()
	{
		if ((lvCerts.getSelectedRowCount() > 0) && (lvCerts.getSelectedRow() < Storage.getCount()))
		{
			TElX509Certificate Cert = Storage.getCertificate(lvCerts.getSelectedRow());
			frmProcessor = new ProcessorForm();
			frmProcessor.CertStorage.clear();
			frmProcessor.CertStorage.add(Cert, true);
			frmProcessor.setModal(true);
			frmProcessor.setVisible(true);
		}
	}
	
	private void lvCerts_SelectedIndexChanged()
	{
		btnRemoveCert.setEnabled(lvCerts.getSelectedRowCount() > 0);
		btnProcessFiles.setEnabled(lvCerts.getSelectedRowCount() > 0);
	}
	
 	private String requestPIN() {
		JTextField jpf = new JTextField();
		int result = JOptionPane.showConfirmDialog(null, jpf, "Please enter your PIN", JOptionPane.OK_CANCEL_OPTION);
		if (result == 0)
			return new String(jpf.getText());
		return "";
	}
}
