import java.awt.BorderLayout;
import java.awt.FlowLayout;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;
import javax.swing.JLabel;
import javax.swing.JTextField;

import SecureBlackbox.Base.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;


public class ProcessorForm extends JDialog {

	private static final long serialVersionUID = 1L;
	private final JPanel contentPanel = new JPanel();
	private JTextField edInputFile;
	private JTextField edOutputFile;

	public TElMemoryCertStorage CertStorage;
	private TElMessageDecryptor Decryptor;
    private TElMessageEncryptor Encryptor;
	private TElMessageSigner Signer;
	private TElMessageVerifier Verifier;
	
	/**
	 * Launch the application.
	 */
	public static void main(String[] args) {
		try {
			ProcessorForm dialog = new ProcessorForm();
			dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
			dialog.setVisible(true);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Create the dialog.
	 */
	public ProcessorForm() {
		setResizable(false);
		setTitle("File processor");
		setBounds(100, 100, 416, 218);
		getContentPane().setLayout(new BorderLayout());
		contentPanel.setBorder(new EmptyBorder(5, 5, 5, 5));
		getContentPane().add(contentPanel, BorderLayout.CENTER);
		contentPanel.setLayout(null);
		{
			JLabel lblInputFile = new JLabel("Input File:");
			lblInputFile.setBounds(10, 21, 179, 14);
			contentPanel.add(lblInputFile);
		}
		
		edInputFile = new JTextField();
		edInputFile.setBounds(10, 40, 292, 20);
		contentPanel.add(edInputFile);
		edInputFile.setColumns(10);
		
		JButton btnBrowse = new JButton("Browse");
		btnBrowse.addMouseListener(new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent e) {
				edInputFile.setText(getOpenFileName());
			}
		});
		btnBrowse.setBounds(312, 39, 91, 23);
		contentPanel.add(btnBrowse);
		
		JLabel lblOutputFile = new JLabel("Output File:");
		lblOutputFile.setBounds(10, 71, 147, 14);
		contentPanel.add(lblOutputFile);
		
		edOutputFile = new JTextField();
		edOutputFile.setColumns(10);
		edOutputFile.setBounds(10, 87, 292, 20);
		contentPanel.add(edOutputFile);
		
		JButton button = new JButton("Browse");
		button.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				edOutputFile.setText(getSaveFileName());				
			}
		});
		button.setBounds(312, 86, 91, 23);
		contentPanel.add(button);
		
		JButton btnSign = new JButton("Sign");
		btnSign.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				signClick();
			}
		});
		btnSign.setBounds(10, 118, 91, 23);
		contentPanel.add(btnSign);
		
		JButton btnVerify = new JButton("Verify");
		btnVerify.addMouseListener(new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent e) {
				verifyClick();
			}
		});
		btnVerify.setBounds(111, 118, 91, 23);
		contentPanel.add(btnVerify);
		
		JButton btnEncrypt = new JButton("Encrypt");
		btnEncrypt.addMouseListener(new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent e) {
				encryptClick();
			}
		});
		btnEncrypt.setBounds(212, 118, 91, 23);
		contentPanel.add(btnEncrypt);
		
		JButton btnDecrypt = new JButton("Decrypt");
		btnDecrypt.addMouseListener(new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent e) {
				decryptClick();
			}
		});
		btnDecrypt.setBounds(312, 118, 91, 23);
		contentPanel.add(btnDecrypt);
		{
			JPanel buttonPane = new JPanel();
			buttonPane.setLayout(new FlowLayout(FlowLayout.RIGHT));
			getContentPane().add(buttonPane, BorderLayout.SOUTH);
			{
				JButton cancelButton = new JButton("Close");
				cancelButton.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent arg0) {
						clear();
						setVisible(false);
					}
				});
				cancelButton.setActionCommand("Cancel");
				buttonPane.add(cancelButton);
			}
		}
		
		init();
	}

	protected void decryptClick() {
		byte[] InBuffer, OutBuffer;
		TElFileStream Stream;
		int InSize, res;
		TSBInteger OutSize = new TSBInteger();
		Stream = new TElFileStream(edInputFile.getText(), "r", true);
		try
		{
			InSize = (int)Stream.getLength();
			InBuffer = new byte[InSize];
			Stream.read(InBuffer, 0, InSize);
		}
		finally
		{
			Stream.Free();
		}
		OutSize.value = InSize + 16384;
		OutBuffer  = new byte[OutSize.value];
		res = Decryptor.decrypt(InBuffer, OutBuffer, OutSize);
		if (res == SBMessages.SB_MESSAGE_ERROR_NO_CERTIFICATE)
		{
			showErrorMessage("Certificate, necessary to decrypt the file, was not found.");
			return;
		}
		else
			if ((res == SBMessages.SB_MESSAGE_ERROR_KEY_DECRYPTION_FAILED) || (res == SBMessages.SB_MESSAGE_ERROR_CONTENT_DECRYPTION_FAILED))
		{
			showErrorMessage("The file could not be decrypted. Most likely the file was changed or is corrupt.");
			return;
		}
		else
			if (res != 0)
		{
			showErrorMessage(String.format("Error {%d} when decrypting the file", res));
			return;
		}
		Stream = new TElFileStream(edOutputFile.getText(), "rw", true);
		try
		{
			Stream.write(OutBuffer, 0, OutSize.value);
		}
		finally
		{
			Stream.Free();
		}
		showMessage("The file has been succesfully decrypted and saved to output file.");
	}

	protected void encryptClick() {
		byte[] InBuffer, OutBuffer;
		TElFileStream Stream;
		int InSize, res;
		TSBInteger OutSize = new TSBInteger();
		Stream = new TElFileStream(edInputFile.getText(), "r", true);
		try
		{
			InSize = (int)Stream.getLength();
			InBuffer = new byte[InSize];
			Stream.read(InBuffer, 0, InSize);
		}
		finally
		{
			Stream.Free();
		}
		OutSize.value = InSize + 16384;
		OutBuffer  = new byte[OutSize.value];
		res = Encryptor.encrypt(InBuffer, OutBuffer, OutSize);
		if (res != 0)
		{
			showErrorMessage(String.format("Error {%d} when encrypting the file", res));
			return;
		}
		Stream = new TElFileStream(edOutputFile.getText(), "rw", true);
		try
		{
			Stream.write(OutBuffer, 0, OutSize.value);
		}
		finally
		{
			Stream.Free();
		}
		showMessage("The file has been succesfully encrypted");
	}

	protected void verifyClick() {
		byte[] InBuffer, OutBuffer;
		TElFileStream Stream;
		int InSize, res;
		TSBInteger OutSize = new TSBInteger();
		Stream = new TElFileStream(edInputFile.getText(), "r", true);
		try
		{
			InSize = (int)Stream.getLength();
			InBuffer = new byte[InSize];
			Stream.read(InBuffer, 0, InSize);
		}
		finally
		{
			Stream.Free();
		}
		OutSize.value = InSize + 16384;
		OutBuffer  = new byte[OutSize.value];
		res = Verifier.verify(InBuffer, OutBuffer, OutSize);
		if (res == SBMessages.SB_MESSAGE_ERROR_NO_CERTIFICATE)
		{
			showMessage("The file was signed using certificate, different from the one you specified");
			return;
		}
		else
			if ((res == SBMessages.SB_MESSAGE_ERROR_INVALID_SIGNATURE) || (res == SBMessages.SB_MESSAGE_ERROR_INVALID_DIGEST))
		{
			showErrorMessage("The signature of the file is invalid. This means that the file has been modified or is corrupt.");
			return;
		}
		else
			if (res != 0)
		{
			showErrorMessage(String.format("Error {0} when verifying the file", res));
			return;
		}
		Stream = new TElFileStream(edOutputFile.getText(), "rw", true);
		try
		{
			Stream.write(OutBuffer, 0, OutSize.value);
		}
		finally
		{
			Stream.Free();
		}
		if (Verifier.getCertificates().findByHashSHA1(CertStorage.getCertificate(0).getHashSHA1()) != -1)
			showMessage("The file has been succesfully verified and saved to output file.");
		else
			showMessage("The file has been verified using internally contained certificate (i.e. not the one you selected).");		
	}

    private void showErrorMessage(String msg) {
    	JOptionPane.showMessageDialog(null, msg, getTitle(), JOptionPane.ERROR_MESSAGE);
	}
	
	protected void signClick() {
		byte[] InBuffer, OutBuffer;
		TElFileStream Stream;
		int InSize, res;
		TSBInteger OutSize = new TSBInteger();
		Stream = new TElFileStream(edInputFile.getText(), "r", true);
		try
		{
			InSize = (int)Stream.getLength();
			InBuffer = new byte[InSize];
			Stream.read(InBuffer, 0, InSize);
		}
		finally
		{
			Stream.Free();
		}
		OutSize.value = InSize + 16384;
		OutBuffer  = new byte[OutSize.value];
		res = Signer.sign(InBuffer, OutBuffer, OutSize, false);
		if (res != 0)
		{
			showErrorMessage(String.format("Error {%d} when signing the file", res));
			return;
		}
		Stream = new TElFileStream(edOutputFile.getText(), "rw", true);
		try
		{
			Stream.write(OutBuffer, 0, OutSize.value);
		}
		finally
		{
			Stream.Free();
		}
		showMessage("The file has been succesfully signed");
	}

	private void showMessage(String msg) {
    	JOptionPane.showMessageDialog(null, msg, getTitle(), JOptionPane.INFORMATION_MESSAGE);
	}
	
	private String getSaveFileName() {
		JFileChooser fc = new JFileChooser();
		
	    int returnVal = fc.showSaveDialog(this);
		if (returnVal == JFileChooser.APPROVE_OPTION)
	        return fc.getSelectedFile().getPath();

	    return "";
	}
	
	String getOpenFileName(){
		JFileChooser fc = new JFileChooser();

	    int returnVal = fc.showOpenDialog(this);
		if (returnVal == JFileChooser.APPROVE_OPTION)
	        return fc.getSelectedFile().getPath();

	    return "";
	}
	
	private void init() {
		CertStorage = new TElMemoryCertStorage();
		Decryptor = new TElMessageDecryptor();
		Encryptor = new TElMessageEncryptor();
		Signer = new TElMessageSigner();
		Verifier = new TElMessageVerifier();

		Decryptor.setCertStorage(CertStorage);
		Encryptor.setCertStorage(CertStorage);
		Encryptor.setUseOAEP(false);
		Encryptor.setBitsInKey(128);
		Encryptor.setAlgorithm(SBConstants.SB_ALGORITHM_CNT_AES128);
		Signer.setCertStorage(CertStorage);
		Signer.setUsePSS(false);
		Verifier.setCertStorage(CertStorage);		
	}
	
	private void clear() {
		CertStorage.Free();
		Decryptor.Free();
		Encryptor.Free();
		Signer.Free();
		Verifier.Free();
	}
}
