

import java.awt.EventQueue;

import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.border.TitledBorder;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JTextField;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JPasswordField;

import SecureBlackbox.Base.*;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;


public class Main extends JDialog {

	private static final long serialVersionUID = 1L;
	private JTextField tbInputFile;
	private JTextField tbOutputFile;
	private JTextField tbKeyFile;
	private JPasswordField tbPassphrase;
	private JComboBox<String> comboAlgorithm;
	private JComboBox<String> comboInputEncoding;
	private JComboBox<String> comboOutputEncoding;
	private JComboBox<String> comboKeyContainerType;

	/**
	 * Launch the application.
	 */
	public static void main(String[] args) {
		EventQueue.invokeLater(new Runnable() {
			public void run() {
				try {
					Main dialog = new Main();
					dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
					dialog.setVisible(true);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}

	/**
	 * Create the dialog.
	 */
	public Main() {
		setTitle("ElPublicKeyCrypto signing with detached signature demo");
		setBounds(100, 100, 549, 377);
		getContentPane().setLayout(null);
		
		JPanel panel = new JPanel();
		panel.setBorder(new TitledBorder(null, "Settings", TitledBorder.LEADING, TitledBorder.TOP, null, null));
		panel.setBounds(10, 11, 523, 297);
		getContentPane().add(panel);
		panel.setLayout(null);
		
		JLabel lblInputFilename = new JLabel("Input filename:");
		lblInputFilename.setBounds(10, 22, 246, 14);
		panel.add(lblInputFilename);
		
		tbInputFile = new JTextField();
		tbInputFile.setBounds(10, 47, 386, 20);
		panel.add(tbInputFile);
		tbInputFile.setColumns(10);
		
		JButton btnBrowseInput = new JButton("Browse");
		btnBrowseInput.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				tbInputFile.setText(getOpenFileName());
			}
		});
		btnBrowseInput.setBounds(422, 46, 91, 23);
		panel.add(btnBrowseInput);
		
		JLabel lblOutputFilename = new JLabel("Output filename:");
		lblOutputFilename.setBounds(10, 78, 174, 14);
		panel.add(lblOutputFilename);
		
		JButton buttonBrowseOutput = new JButton("Browse");
		buttonBrowseOutput.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				tbOutputFile.setText(getSaveFileName());
			}
		});
		buttonBrowseOutput.setBounds(422, 96, 91, 23);
		panel.add(buttonBrowseOutput);
		
		tbOutputFile = new JTextField();
		tbOutputFile.setColumns(10);
		tbOutputFile.setBounds(10, 97, 386, 20);
		panel.add(tbOutputFile);
		
		JLabel lblSignatureAlgorithm = new JLabel("Signature algorithm:");
		lblSignatureAlgorithm.setBounds(10, 131, 136, 14);
		panel.add(lblSignatureAlgorithm);
		
		JLabel lblInputEncoding = new JLabel("Input encoding:");
		lblInputEncoding.setBounds(155, 131, 86, 14);
		panel.add(lblInputEncoding);
		
		JLabel lblOutputEncoding = new JLabel("Output encoding:");
		lblOutputEncoding.setBounds(289, 131, 107, 14);
		panel.add(lblOutputEncoding);
		
		comboAlgorithm = new JComboBox<String>();
		comboAlgorithm.setModel(new DefaultComboBoxModel<String>(new String[] {"RSA", "DSA"}));
		comboAlgorithm.setBounds(10, 151, 97, 22);
		panel.add(comboAlgorithm);
		
		comboInputEncoding = new JComboBox<String>();
		comboInputEncoding.setModel(new DefaultComboBoxModel<String>(new String[] {"Binary", "Base64"}));
		comboInputEncoding.setBounds(155, 151, 91, 22);
		panel.add(comboInputEncoding);
		
		comboOutputEncoding = new JComboBox<String>();
		comboOutputEncoding.setModel(new DefaultComboBoxModel<String>(new String[] {"Binary", "Base64"}));
		comboOutputEncoding.setBounds(289, 151, 91, 22);
		panel.add(comboOutputEncoding);
		
		JLabel lblPrivateKeyContainer = new JLabel("Private key container file:");
		lblPrivateKeyContainer.setBounds(10, 184, 174, 14);
		panel.add(lblPrivateKeyContainer);
		
		tbKeyFile = new JTextField();
		tbKeyFile.setColumns(10);
		tbKeyFile.setBounds(10, 203, 386, 20);
		panel.add(tbKeyFile);
		
		JButton buttonBrowsePrivateKey = new JButton("Browse");
		buttonBrowsePrivateKey.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				tbKeyFile.setText(getOpenFileName());
			}
		});
		buttonBrowsePrivateKey.setBounds(422, 202, 91, 23);
		panel.add(buttonBrowsePrivateKey);
		
		JLabel lblPassword = new JLabel("Password");
		lblPassword.setBounds(10, 234, 136, 14);
		panel.add(lblPassword);
		
		tbPassphrase = new JPasswordField();
		tbPassphrase.setBounds(10, 259, 136, 20);
		panel.add(tbPassphrase);
		
		JLabel lblKeyContainerFile = new JLabel("Key container type:");
		lblKeyContainerFile.setBounds(155, 234, 162, 14);
		panel.add(lblKeyContainerFile);
		
		comboKeyContainerType = new JComboBox<String>();
		comboKeyContainerType.setModel(new DefaultComboBoxModel<String>(new String[] {"Generic private key", "X.509 certificate"}));
		comboKeyContainerType.setBounds(155, 258, 162, 22);
		panel.add(comboKeyContainerType);
		
		JButton btnSign = new JButton("Sign");
		btnSign.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (!new File(tbInputFile.getText()).exists())
					showMessage("Source file not found");
				else if (tbOutputFile.getText().compareTo("") == 0)
					showMessage("Please, select output file");
				else if (!new File(tbKeyFile.getText()).exists())
					showMessage("Key or certificate file not found");
				else
					DoSignDetached();				
			}
		});
		btnSign.setBounds(166, 319, 91, 23);
		getContentPane().add(btnSign);
		
		JButton btnCancel = new JButton("Cancel");
		btnCancel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				dispose();
				System.exit(0);
			}
		});
		btnCancel.setBounds(267, 319, 91, 23);
		getContentPane().add(btnCancel);
		
		init();
	}

	private void init() {
		#error Please pick the evaluation license key from <SecureBlackbox>\LicenseKey.txt file and place it here. If the evaluation key expires, you can request an extension using the form on https://www.eldos.com/sbb/keyreq/
SBUtils.setLicenseKey("<put the key here>");
	}

	private void showMessage(String msg) {
		JOptionPane.showMessageDialog(null, msg, getTitle(), JOptionPane.INFORMATION_MESSAGE);		
	}

	private void showMessage(String msg, String cap) {
		JOptionPane.showMessageDialog(null, msg, cap, JOptionPane.INFORMATION_MESSAGE);		
	}
	
	private void DoSignDetached()
	{
		TElPublicKeyCrypto Crypto = null;
		FileInputStream streamInput;
		FileOutputStream streamOutput;
		FileInputStream streamKey;
		boolean keyLoaded = false;
		TElPublicKeyMaterial KeyMaterial = null;
		TElX509Certificate cert;
		int certType; 
		String errorMsg = "";
		int r;
		
		switch (comboAlgorithm.getSelectedIndex())
		{
			// RSA
			case 0: 
			{
				Crypto = new TElRSAPublicKeyCrypto();
				KeyMaterial = new TElRSAKeyMaterial();
				break;
			}
			// DSA
			case 1: 
			{
				Crypto = new TElDSAPublicKeyCrypto();
				KeyMaterial = new TElDSAKeyMaterial();
				break;
			}
		}
		if (comboInputEncoding.getSelectedIndex() == 0) 
		{
			Crypto.setInputEncoding(TSBPublicKeyCryptoEncoding.pkeBinary);
		} 
		else 
		{
			Crypto.setInputEncoding(TSBPublicKeyCryptoEncoding.pkeBase64);
		}
		if (comboOutputEncoding.getSelectedIndex() == 0) 
		{
			Crypto.setOutputEncoding(TSBPublicKeyCryptoEncoding.pkeBinary);
		} 
		else 
		{
			Crypto.setOutputEncoding(TSBPublicKeyCryptoEncoding.pkeBase64);
		}
		try 
		{
			if (comboKeyContainerType.getSelectedIndex() == 1)
			{
				cert = new TElX509Certificate();
				streamKey = new FileInputStream(tbKeyFile.getText());
				try
				{
					certType = TElX509Certificate.detectCertFileFormat(streamKey);
					if (certType == 1) // cfDER
					{
						try 
						{
							cert.loadFromStream(streamKey, 0);
							keyLoaded = true;
						} 
						catch(Exception e) 
						{
							errorMsg = e.getMessage();
						}
					}
					else if (certType == 2) // cfPEM
					{
						r = cert.loadFromStreamPEM(streamKey, new String(tbPassphrase.getPassword()), 0);
						if (r != 0) 
						{
							errorMsg = "PEM read error: " + r;
						} 
						else 
						{
							keyLoaded = true;
						}
					}
					else if (certType == 3) // cfPFX
					{
						r = cert.loadFromStreamPFX(streamKey, new String(tbPassphrase.getPassword()), 0);
						if (r != 0) 
						{
							errorMsg = "PFX read error: " + r;
						} 
						else 
						{
							keyLoaded = true;
						}
					}
					else
					{
						errorMsg = "Unknown certificate file format";
					}
					if (!keyLoaded) 
					{
						throw (new Exception(errorMsg));
					}
				} 
				finally
				{ 
					streamKey.close();
				}
				KeyMaterial.assign(cert.getKeyMaterial());
			}
			else
			{
				streamKey = new FileInputStream(tbKeyFile.getText());
				try
				{
					if (KeyMaterial instanceof TElRSAKeyMaterial) 
					{
						((TElRSAKeyMaterial)(KeyMaterial)).setPassphrase(new String(tbPassphrase.getPassword()));
					} 
					else if (KeyMaterial instanceof TElDSAKeyMaterial) 
					{
						((TElDSAKeyMaterial)(KeyMaterial)).setPassphrase(new String(tbPassphrase.getPassword()));
					}
					KeyMaterial.loadSecret(streamKey, 0);
				}
				finally
				{
					streamKey.close();
				}
			}

			Crypto.setKeyMaterial(KeyMaterial);
			streamInput = new FileInputStream(tbInputFile.getText());
			try
			{
				streamOutput = new FileOutputStream(tbOutputFile.getText());
				try
				{
					Crypto.signDetached(streamInput, streamOutput, 0);
				}
				finally
				{
					streamOutput.close();
				}
			}
			finally
			{
				streamInput.close();
			}
			showMessage("The file was signed successfully", "Succes");
		} 
		catch(Exception e) 
		{
			showErrorMessage(e.getMessage());
		}
	}
	
	private void showErrorMessage(String msg) {
		JOptionPane.showMessageDialog(null, msg, "Error", JOptionPane.ERROR_MESSAGE);
	}
	
	protected String getSaveFileName() {
		JFileChooser fc = new JFileChooser();
			    
	    int returnVal = fc.showSaveDialog(this);
		if (returnVal == JFileChooser.APPROVE_OPTION)
	        return fc.getSelectedFile().getPath();

	    return "";
	}
	
	protected String getOpenFileName() {
		JFileChooser fc = new JFileChooser();
			    
	    int returnVal = fc.showOpenDialog(this);
		if (returnVal == JFileChooser.APPROVE_OPTION)
	        return fc.getSelectedFile().getPath();

	    return "";
	}
	
}
