package com.eldos.idp;

import SecureBlackbox.Base.*;
import SecureBlackbox.SAMLCommon.*;
import SecureBlackbox.SAMLIDP.TElSAMLServiceProviderInfo;
import SecureBlackbox.XML.TElXMLDOMDocument;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

public class SSOServlet extends HttpServlet {

    private List<TElSAMLServiceProviderInfo> spList = new ArrayList<>();
    private String prefferedSSOResponseBinding;
    private String idpIssuer, signCertPath;
    private boolean signResponses;

    @Override
    public void init(ServletConfig config) throws ServletException {
        super.init(config);

        String spCountStr = config.getInitParameter("sp_count");
        int spCount = Integer.parseInt(spCountStr);

        TElSAMLServiceProviderInfo spInfo;
        for (int i = 1; i <= spCount; i++) {
            String meta = config.getInitParameter("sp" + i + "_metadata");
            if (meta == null)
                break;

            spInfo = new TElSAMLServiceProviderInfo();
            spInfo.loadMetadata(meta);
            spList.add(spInfo);
        }

        if (spList.isEmpty())
            throw new RuntimeException("No service providers configured!");

        prefferedSSOResponseBinding = config.getInitParameter("preffered_sso_resp_binding");
        idpIssuer = config.getInitParameter("issuer");
        signCertPath = config.getInitParameter("sign_cert");

        String s = config.getInitParameter("sign_responses");
        signResponses = Boolean.parseBoolean(s);
    }

    @Override
    protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        TElSAMLRedirectBinding binding = new TElSAMLRedirectBinding();
        binding.setVerifySignatures(false);

        String body = "?" + req.getQueryString();
        String saml = binding.loadRequest(body);
        String state = binding.getRelayState();

        loadAndProcessRequest(req, resp, saml, state);
    }

    @Override
    protected void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        TElSAMLPOSTBinding binding = new TElSAMLPOSTBinding();
        binding.setMode(TSBSAMLPOSTBindingMode.spmServer);

        String body = streamToStr(req.getInputStream());
        String saml = binding.loadRequest(body);
        String state = binding.getRelayState();

        loadAndProcessRequest(req, resp, saml, state);
    }

    private void loadAndProcessRequest(HttpServletRequest req, HttpServletResponse resp,
                                       String samlBody, String state) throws ServletException, IOException {
        HttpSession session = req.getSession(true);
        String login = (String) session.getAttribute("login");
        boolean loggedIn = (session.getAttribute("loggedIn") != null) ? (Boolean) session.getAttribute("loggedIn") : false;

        TElXMLDOMDocument doc = new TElXMLDOMDocument();
        byte[] samlBytes = SBStrUtils.strToUTF8(samlBody);
        TElMemoryStream ms = new TElMemoryStream(samlBytes, 0, samlBytes.length);
        doc.loadFromStream(ms);

        if (loggedIn) {
            Utils.processRequest(req, resp, doc, login, state, spList, prefferedSSOResponseBinding,
                    idpIssuer, signCertPath, signResponses);
        } else {
            session.setAttribute("requestBody", doc);
            session.setAttribute("requestState", state);
            session.setAttribute("spList", spList);
            session.setAttribute("prefferedSSOResponseBinding", prefferedSSOResponseBinding);
            session.setAttribute("idpIssuer", idpIssuer);
            session.setAttribute("signCertPath", signCertPath);
            session.setAttribute("signResponses", signResponses);

            RequestDispatcher view = req.getRequestDispatcher("/WEB-INF/login.html");
            view.forward(req, resp);
        }
    }

    private String streamToStr(InputStream is) throws IOException {
        StringBuilder sb = new StringBuilder();
        byte[] buf = new byte[1024];
        int r;

        do {
            if ((r = is.read(buf)) > 0)
                sb.append(new String(buf, 0, r, "utf-8"));
        } while (r > 0);

        return sb.toString();
    }
}
