import java.io.*;

import org.freepascal.rtl.TObject;

import SecureBlackbox.SFTPCommon.*;


public class SFTPFileStream extends TObject {
	String m_path;
	RandomAccessFile m_stream;
	int m_errCode;
	
	public SFTPFileStream(String path, int modes) {
		try {
			m_path = path;
			m_stream = new RandomAccessFile(path, getStringMode(modes));
		} catch (Exception e) {
			e.printStackTrace();
			Logger.Log(e.getMessage());
		}
	}
	
	private String getStringMode(int modes) throws Exception {
		String result = "";
		if (((modes & SBSftpCommon.fmRead) == SBSftpCommon.fmRead) &&
				((modes & SBSftpCommon.fmWrite) == SBSftpCommon.fmWrite))
		{
			result = "rw";
		} 
		else if ((modes & SBSftpCommon.fmWrite) == SBSftpCommon.fmWrite) 
		{
			result = "rw";
		}
		else 
		{
			result = "r";
		}
		
		switch (checkAccessMode(m_path, modes)){
			case 1:
				if (result.compareTo("r") == 0)
					result = "rw";
		}

		return result;
	}

	public String getFileName() {
		return new File(m_path).getName();
	}

	public String getPath() {
		return m_path;
	}

	public long getLength() throws IOException {
		return m_stream.length();
	}

	public void setPosition(long pos) throws IOException {
		m_stream.seek(pos);
	}

	public int getErrorCode() {
		return m_errCode;
	}

	protected int checkAccessMode(String realpath, int modes) throws Exception {
		int result = -1;
		if (((modes & SBSftpCommon.fmCreate) == SBSftpCommon.fmCreate) &&
				((modes & SBSftpCommon.fmTruncate) == SBSftpCommon.fmTruncate))
			{
				result = 1;
			} 
			else if (((modes & SBSftpCommon.fmCreate) == SBSftpCommon.fmCreate) &&
				((modes & SBSftpCommon.fmExcl) == SBSftpCommon.fmExcl))
			{
				if (new File(realpath).exists()) 
				{
					m_errCode = SBSftpCommon.SSH_ERROR_FILE_ALREADY_EXISTS;
					throw new Exception("Cannot open file");
				} 
				else 
				{
					result = 1;
				}
			} 
			else 
			{
				if ((modes & SBSftpCommon.fmCreate) == SBSftpCommon.fmCreate) 
				{
					result = 1;
				} 
				else 
				{
					result = 0;
				}
			}
		return result;
	}

	public void write(byte[] buffer, int offset, int count) {
		try {
			m_stream.write(buffer, offset, count);
		} catch (IOException e) {
			e.printStackTrace();
			Logger.Log(e.getMessage());
		}
	}

	public void close() {
		try {
			m_stream.close();
		} catch (IOException e) {
			e.printStackTrace();
			Logger.Log(e.getMessage());
		}
	}

	public int read(byte[] buffer, int bufferOffset, int count) {
		try {
			return m_stream.read(buffer, bufferOffset, count);
		} catch (IOException e) {
			e.printStackTrace();
			Logger.Log(e.getMessage());
		}
		return 0;
	}

}
