

import java.awt.EventQueue;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JDialog;
import javax.swing.JMenuBar;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.io.File;
import java.net.URL;
import java.util.Date;

import javax.swing.JTable;

import SecureBlackbox.Base.SBUtils;
import SecureBlackbox.DAVCommon.SBWebDAVCommon;
import SecureBlackbox.DAVServer.TElWebDAVMemoryLockList;
import SecureBlackbox.HTTPCommon.TElHTTPUtils;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import javax.swing.JToolBar;
import javax.swing.JButton;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;


public class Main extends JDialog {

	private static final long serialVersionUID = 1L;
	private JTable tableReq;
	private JTable tbLog;
	private ServerThread m_serverThread;
	private TElWebDAVMemoryLockList m_lockList;
	
	/**
	 * Launch the application.
	 */
	public static void main(String[] args) {
		EventQueue.invokeLater(new Runnable() {
			public void run() {
				try {
					Main dialog = new Main();
					dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
					dialog.setVisible(true);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}

	/**
	 * Create the dialog.
	 */
	public Main() {
		setTitle("EldoS WebDAV(S) Server Demo");
		setBounds(100, 100, 586, 350);
		
		JMenuBar menuBar = new JMenuBar();
		setJMenuBar(menuBar);
		
		JMenu mnServer = new JMenu("Server");
		menuBar.add(mnServer);
		
		JMenuItem mntmStart = new JMenuItem("Start");
		mntmStart.addMouseListener(new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent e) {
				Start();
			}
		});
		mnServer.add(mntmStart);
		
		JMenuItem mntmStop = new JMenuItem("Stop");
		mntmStop.addMouseListener(new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent e) {
				Stop();
			}
		});
		mnServer.add(mntmStop);
		
		JMenuItem mntmExit = new JMenuItem("Exit");
		mntmExit.addMouseListener(new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent e) {
				close();
			}
		});
		mnServer.add(mntmExit);
		
		JMenu mnHelp = new JMenu("Help");
		menuBar.add(mnHelp);
		
		JMenuItem mntmAbout = new JMenuItem("About");
		mntmAbout.addMouseListener(new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent e) {
				showMessage("SecureBlackbox TElWebDAVServer server demo.\n" +
						"Please visit http://www.eldos.com/sbb/ for more information!");
			}
		});
		mnHelp.add(mntmAbout);
		
		JScrollPane scrollPane = new JScrollPane();
		scrollPane.setPreferredSize(new Dimension(4, 100));
		scrollPane.setBounds(new Rectangle(0, 0, 0, 50));
		scrollPane.setSize(new Dimension(0, 50));
		getContentPane().add(scrollPane, BorderLayout.SOUTH);
		
		tbLog = new JTable(new LogTableModel());
		tbLog.setFillsViewportHeight(true);
		scrollPane.setViewportView(tbLog);
		
		JScrollPane scrollPane_1 = new JScrollPane();
		getContentPane().add(scrollPane_1, BorderLayout.CENTER);
		
		tableReq = new JTable(new ReqTableModel());
		tableReq.setFillsViewportHeight(true);
		scrollPane_1.setViewportView(tableReq);
		
		JToolBar toolBar = new JToolBar();
		toolBar.setFloatable(false);
		getContentPane().add(toolBar, BorderLayout.NORTH);
		
		JButton btnStart = new JButton(" ");
		btnStart.setIcon(getIcon("start"));		
		btnStart.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				Start();
			}
		});
		toolBar.add(btnStart);
		
		JButton btnStop = new JButton(" ");
		btnStop.setIcon(getIcon("stop"));
		btnStop.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				Stop();
			}
		});
		toolBar.add(btnStop);

		init();
	}


	private Icon getIcon(String name) {
		URL resPath = getClass().getResource(name + ".ico");
		if (resPath == null) return new ImageIcon();
		return new ImageIcon(resPath);
	}
	
	protected void close() {
		if (m_serverThread != null)
		{
			m_serverThread.Dispose();
		}
		
		dispose();
	}

	private void init() {
		#error Please pick the evaluation license key from <SecureBlackbox>\LicenseKey.txt file and place it here. If the evaluation key expires, you can request an extension using the form on https://www.eldos.com/sbb/keyreq/
		SBUtils.setLicenseKey("<put the key here>");	
	}

	boolean CreateEmptyAttributesDB(String filePath)
	{
		try
		{
			File attrFile = new File(filePath);
			if(!attrFile.exists()) attrFile.createNewFile();
			JIniFile ini = new JIniFile(filePath);

			ini.WriteString("/", "<ns>DAV:</ns><name>displayname</name>", "/");
			ini.WriteString("/", "<ns>DAV:</ns><name>getcontentlanguage</name>", "en");
			ini.WriteString("/", "<ns>DAV:</ns><name>getcontenttype</name>", "httpd/unix-directory");
			ini.WriteString("/", "<ns>DAV:</ns><name>getetag</name>", "5e77061a9b955ac03d9a574a8a68d8a58005949fa4fdd3d9e38dbd263028ffe8");
			ini.WriteString("/", "<ns>DAV:</ns><name>resourcetype</name>", "<collection/>");
			ini.WriteString("/", "<ns>DAV:</ns><name>getcontentlength</name>", "0");
			ini.WriteString("/", "<ns>DAV:</ns><name>creationdate</name>", SBWebDAVCommon.dateTimeToISOTime(new Date()));
			ini.WriteString("/", "<ns>DAV:</ns><name>getlastmodified</name>", TElHTTPUtils.dateTimeToHTTPTime(new Date()));
			ini.UpdateFile();
		}
		catch (Exception ex)
		{
			return false;
		}
		return true;
	}

    public boolean IsDirectoryEmpty(String path)
    {
    	File file = new File(path);
    	 
    	if (file.isDirectory()) {
    		if(file.list().length > 0)
    			return false;
    		else
    			return true;
    	}
    	return false;
    }
    
	private void Start()
	{
		if (m_serverThread == null)
		{
			m_lockList = new TElWebDAVMemoryLockList();

			Settings settingsForm = new Settings();
			settingsForm.setModal(true);
			settingsForm.setVisible(true);
			if (settingsForm.isOk())
			{
				if ( ! CreateEmptyAttributesDB(settingsForm.getAttributesDBPath()))
				{
					showErrorMessage("Can not create an empty attributes database!");
					return;
				}
                if ( ! IsDirectoryEmpty(settingsForm.getBasePath()))
                {
                	showErrorMessage("You should select an empty folder as a base path!");
                    return;
                }

				m_serverThread = new ServerThread(new Logger(this), settingsForm.getPort(), settingsForm.getUseSSL(),
                    m_lockList, settingsForm.getDiskAdapter(), settingsForm.getCertStorage(), settingsForm.getUsers());

                AddLogEvent("Server started");
			}
		}
	}
	
    private void showErrorMessage(String msg) {
    	JOptionPane.showMessageDialog(null, msg, getTitle(), JOptionPane.ERROR_MESSAGE);
	}

    private void showMessage(String msg) {
    	JOptionPane.showMessageDialog(null, msg, getTitle(), JOptionPane.INFORMATION_MESSAGE);
	}

	private void Stop()
	{
		if (m_serverThread != null)
		{
			m_serverThread.Dispose();
			m_serverThread = null;

            AddLogEvent("Server stopped");
		}
	}
    
	public void AddLogEvent(String msg) {
		AddLogEvent(msg, false);
	}
	
	public void AddLogEvent(String msg, boolean isError)
	{
		LogTableModel model = (LogTableModel)tbLog.getModel();
		model.addRow(new Date(), msg, isError ? "Eror" : "Info");
	}

	public void AddRequestEvent(String Method, String URL)
	{
		ReqTableModel model = (ReqTableModel)tableReq.getModel();
		model.addRow(Method, URL);
	}
}
