import java.awt.BorderLayout;
import java.awt.FlowLayout;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;

import SecureBlackbox.Base.*;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.border.LineBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import java.awt.Color;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;


public class SelWinCertDialog extends JDialog {

	private static final long serialVersionUID = 1L;
	private final JPanel contentPanel = new JPanel();
	TElWinCertStorage WinCertStorage;
	private TElX509Certificate FCertificate;
	private JTable table;
	private JLabel lblPrivateKey;
	private JLabel lbUsage;
	private JLabel dlbUsage;
	private JLabel dlbPrivateKey;
	private WinCertTableModel model;
	private boolean isOk;
	
	/**
	 * Launch the application.
	 */
	public static void main(String[] args) {
		try {
			SelWinCertDialog dialog = new SelWinCertDialog();
			dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
			dialog.setVisible(true);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Create the dialog.
	 */
	public SelWinCertDialog() {
		setTitle("Select sertificates");
		setBounds(100, 100, 577, 472);
		getContentPane().setLayout(new BorderLayout());
		contentPanel.setBorder(new EmptyBorder(5, 5, 5, 5));
		getContentPane().add(contentPanel, BorderLayout.CENTER);
		contentPanel.setLayout(null);
		
		JLabel lblCertificatesFrommy = new JLabel("Certificates from \"MY\" Windows Storage:");
		lblCertificatesFrommy.setBounds(10, 11, 355, 14);
		contentPanel.add(lblCertificatesFrommy);
		
		JScrollPane scrollPane = new JScrollPane();
		scrollPane.setBounds(10, 31, 549, 249);
		contentPanel.add(scrollPane);
		
		model = new WinCertTableModel();
		table = new JTable(model);
		table.setFillsViewportHeight(true);
		scrollPane.setViewportView(table);
		table.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
			
			public void valueChanged(ListSelectionEvent e) {				
				if (table.getSelectedRowCount() > 0)
					FCertificate = (TElX509Certificate)model.getTag(table.getSelectedRow());
				else
					FCertificate = null;

				UpdateCertInfo();	
			}
		});
		JPanel panel = new JPanel();
		panel.setBorder(new LineBorder(new Color(0, 0, 0)));
		panel.setBounds(10, 291, 549, 110);
		contentPanel.add(panel);
		panel.setLayout(null);
		
		lblPrivateKey = new JLabel("Private key exists:");
		lblPrivateKey.setBounds(10, 11, 121, 14);
		panel.add(lblPrivateKey);
		
		lbUsage = new JLabel("Certificate Usage:");
		lbUsage.setBounds(10, 36, 121, 14);
		panel.add(lbUsage);
		
		dlbUsage = new JLabel(" ");
		dlbUsage.setBounds(131, 36, 408, 63);
		panel.add(dlbUsage);
		
		dlbPrivateKey = new JLabel(" ");
		dlbPrivateKey.setBounds(131, 11, 408, 14);
		panel.add(dlbPrivateKey);
		{
			JPanel buttonPane = new JPanel();
			buttonPane.setLayout(new FlowLayout(FlowLayout.RIGHT));
			getContentPane().add(buttonPane, BorderLayout.SOUTH);
			{
				JButton okButton = new JButton("OK");
				okButton.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						isOk = true;
						setVisible(false);
					}
				});
				okButton.setActionCommand("OK");
				buttonPane.add(okButton);
				getRootPane().setDefaultButton(okButton);
			}
			{
				JButton cancelButton = new JButton("Cancel");
				cancelButton.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						dispose();
					}
				});
				cancelButton.setActionCommand("Cancel");
				buttonPane.add(cancelButton);
			}
		}
		
		init();
	}
	
	private void init() {
		try
		{
	        // Initialize SecureBlackbox JNI library. sbbjni32.dll and sbbjni64.dll must be located in java.library.path
	        // You can also call JNI.initialize("full_path_to_dll") method.
	        JNI.initialize();
	        
			WinCertStorage = new TElWinCertStorage();
			WinCertStorage.getSystemStores().add("MY");
			UpdateCertificates();
			UpdateCertInfo();
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

	public void UpdateCertInfo()
	{
		TElKeyUsageExtension KeyUsage;
		//TElExtendedKeyUsageExtension ExtKeyUsage;
		String s;
		if (FCertificate == null)
		{
			dlbPrivateKey.setText("None");
			dlbUsage.setText("None");
			return;
		}

		if (FCertificate.getPrivateKeyExists())
			dlbPrivateKey.setText("Available");
		else
			dlbPrivateKey.setText("None");

		s = "";
		if ((FCertificate.getExtensions().getIncluded() & SBX509Ext.ceKeyUsage) == SBX509Ext.ceKeyUsage)
		{
			KeyUsage = FCertificate.getExtensions().getKeyUsage();
			if (KeyUsage.getDigitalSignature())
				s = s + "Digital Signitare, ";
			if (KeyUsage.getNonRepudiation())
				s = s + "Non-repudiation, ";
			if (KeyUsage.getKeyEncipherment())
				s = s + "Key Transport, ";
			if (KeyUsage.getDataEncipherment())
				s = s + "Data Encipherment, ";
			if (KeyUsage.getKeyAgreement())
				s = s + "Key Agreement, ";
			if (KeyUsage.getKeyCertSign())
				s = s + "Certificate Signing, ";
			if (KeyUsage.getCRLSign())
				s = s + "CRL Signing, ";
			if (KeyUsage.getEncipherOnly())
				s = s + "Encipher only, ";
			if (KeyUsage.getDecipherOnly())
				s = s + "Decipher only, ";

			if (s.length() > 0)
				s = s.substring(0, s.length() - 2);
		}
		dlbUsage.setText(s);
	}

	public void UpdateCertificates()
	{
		int i;
		TElX509Certificate Cert;
		for (i = 0; i < WinCertStorage.getCount(); i++)
		{
			Cert = WinCertStorage.getCertificate(i);
			model.addRow(Cert);
		}
	}
	
	public TElX509Certificate getSelWinCertificate() {
		return FCertificate;
	}

	public boolean isOK() {
		return isOk;
	}
}
