import SecureBlackbox.ZIP.*;

import javax.swing.*;
import javax.swing.border.EmptyBorder;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;


public class ArchiveOptionsDialog extends JDialog implements ActionListener {
	private static final long serialVersionUID = 9082014456767725674L;
	private final JPanel contentPanel = new JPanel();
	private JTextField textField;
	private JPasswordField textBoxPassword;
	private JPasswordField textBoxPasswordConfirmation;
	
	private boolean isNeedCreate;
	
	JComboBox<String> comboBoxCompressionAlgorithm;
	JComboBox<String> comboBoxCompressionLevel;
	JComboBox<String> comboBoxEncryption;
	JComboBox<String> comboBoxEncryptionAlgorithm;

	JButton btnOK;
	JCheckBox checkBoxCompressDirectory;
	JCheckBox checkBoxEncryptDirectory;
	JLabel labelCompressionLevel;
	JLabel labelEncryptionAlgorithm;
	JLabel labelCompressionAlgorithm;
	JLabel labelPassword;
	JLabel labelPasswordConfirmation;
	
	/**
	 * Launch the application.
	 */
	public static void main(String[] args) {
		try {
			ArchiveOptionsDialog dialog = new ArchiveOptionsDialog();
			dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
			dialog.setVisible(true);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Create the dialog.
	 */
	public ArchiveOptionsDialog() {
		setTitle("Archive properties");
		setBounds(100, 100, 550, 170);
		getContentPane().setLayout(new BorderLayout());
		contentPanel.setBorder(new EmptyBorder(5, 5, 5, 5));
		getContentPane().add(contentPanel, BorderLayout.CENTER);
		contentPanel.setLayout(null);
		
		JLabel lblNewLabel = new JLabel("Archive File");
		lblNewLabel.setBounds(10, 11, 71, 14);
		contentPanel.add(lblNewLabel);
		
		textField = new JTextField();
		textField.setBounds(79, 8, 353, 20);
		contentPanel.add(textField);
		textField.setColumns(10);
		
		JButton btnChoose = new JButton("Choose...");
		btnChoose.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				textField.setText(getFileName());
			}
		});
		btnChoose.setBounds(442, 7, 90, 23);
		contentPanel.add(btnChoose);
		
		JPanel panel = new JPanel();
		panel.setBounds(10, 35, 515, 60);
        panel.setBorder(BorderFactory.createTitledBorder("Compression"));
		contentPanel.add(panel);
		panel.setLayout(null);
		
		labelCompressionAlgorithm = new JLabel("Algorithm");
		labelCompressionAlgorithm.setHorizontalAlignment(SwingConstants.RIGHT);
		labelCompressionAlgorithm.setBounds(13, 25, 66, 14);
		panel.add(labelCompressionAlgorithm);
		
		comboBoxCompressionAlgorithm = new JComboBox<String>();
		comboBoxCompressionAlgorithm.setBounds(100, 21, 125, 22);
		comboBoxCompressionAlgorithm.setModel(new DefaultComboBoxModel<String>(new String[] {"None", "GZip", "BZip2"}));
		panel.add(comboBoxCompressionAlgorithm);
		
		labelCompressionLevel = new JLabel("Level");
		labelCompressionLevel.setHorizontalAlignment(SwingConstants.RIGHT);
		labelCompressionLevel.setBounds(13, 25, 280, 14);
		panel.add(labelCompressionLevel);
		
		comboBoxCompressionLevel = new JComboBox<String>();
		comboBoxCompressionLevel.setBounds(300, 21, 126, 22);
		comboBoxCompressionLevel.setModel(new DefaultComboBoxModel<String>(new String[] {"Fastest", "Low", "Normal", "High"}));
		panel.add(comboBoxCompressionLevel);

		{
			JPanel buttonPane = new JPanel();
			buttonPane.setLayout(new FlowLayout(FlowLayout.RIGHT));
			getContentPane().add(buttonPane, BorderLayout.SOUTH);
			{
				btnOK = new JButton("OK");
				btnOK.setEnabled(false);
				btnOK.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						isNeedCreate = true;
						setVisible(false);
					}
				});
				btnOK.setActionCommand("OK");
				buttonPane.add(btnOK);
				getRootPane().setDefaultButton(btnOK);
			}
			{
				JButton cancelButton = new JButton("Cancel");
				cancelButton.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						dispose();
					}
				});
				cancelButton.setActionCommand("Cancel");
				buttonPane.add(cancelButton);
			}
		}
		addFieldsListener(this);
		refreshControlsVisibility();
	}
	
	public void addFieldsListener(final ActionListener l) {
		comboBoxCompressionAlgorithm.addActionListener(l);
		addTextDocumentLisner(textField, l);
	}		 
	
	public void addTextDocumentLisner(final JTextField txt, final ActionListener l) {
		txt.getDocument().addDocumentListener(new DocumentListener() {
			public void changedUpdate(DocumentEvent e) {
				l.actionPerformed(new ActionEvent(txt, 0, "changed"));
			}

			public void insertUpdate(DocumentEvent arg0) {
				l.actionPerformed(new ActionEvent(txt, 1, "insert"));
			}

			public void removeUpdate(DocumentEvent e) {			
				l.actionPerformed(new ActionEvent(txt, 2, "remove"));
			}
		});
	}
	
	String getNewArchiveFullPath() {
		return textField.getText();
	}
	
	String getFileName(){
		JFileChooser fc = new JFileChooser();
		File f = new File("archive.tar");
		fc.setSelectedFile(f);

	    int returnVal = fc.showSaveDialog(this);
		if (returnVal == JFileChooser.APPROVE_OPTION)
	        return fc.getSelectedFile().getPath();

	    return "";
	}

	public boolean getIsCreateArch() {
		return isNeedCreate && textField.getText().length() > 0;
	}

	public TSBTarCompressMethod getCompressionAlg() {
		switch (comboBoxCompressionAlgorithm.getSelectedIndex())
		{
			case 1 : return TSBTarCompressMethod.cmGZip;
			case 2 : return TSBTarCompressMethod.cmBZip2;
			default : return TSBTarCompressMethod.cmNone;
		}
	}

	public int getCompressionLevel() {
		switch (comboBoxCompressionLevel.getSelectedIndex())
		{
			case 0 : return 1;
			case 1 : return 3;
			case 2 : return 6;
			default : return 9;
		}
	}

	public void actionPerformed(ActionEvent e) {
		refreshControlsVisibility();
		
		if (getNewArchiveFullPath().length() == 0)
		{
			btnOK.setEnabled(false);
			return;
		} else
		{
			btnOK.setEnabled(true);
		}
	}
	
	private void refreshControlsVisibility()
	{
		if (comboBoxCompressionAlgorithm.getSelectedIndex() == 0)
		{
			labelCompressionLevel.setVisible(false);
			comboBoxCompressionLevel.setVisible(false);
		}
		else
		{
			labelCompressionLevel.setVisible(true);
			comboBoxCompressionLevel.setVisible(true);
		}
	}
}
